@@ -5,7 +5,6 @@ mod seatbelt;
 
 use std::path::PathBuf;
 
-use codex_common::CliConfigOverrides;
 use codex_core::config::Config;
 use codex_core::config::ConfigOverrides;
 use codex_core::exec_env::create_env;
@@ -14,6 +13,7 @@ use codex_core::landlock::spawn_command_under_linux_sandbox;
 use codex_core::seatbelt::spawn_command_under_seatbelt;
 use codex_core::spawn::StdioPolicy;
 use codex_protocol::config_types::SandboxMode;
+use codex_utils_cli::CliConfigOverrides;
 
 use crate::LandlockCommand;
 use crate::SeatbeltCommand;
