@@ -63,6 +63,8 @@ pub enum ResponseEvent {
     Completed {
         response_id: String,
         token_usage: Option<TokenUsage>,
+        /// Whether the client can append more items to a long-running websocket response.
+        can_append: bool,
     },
     OutputTextDelta(String),
     ReasoningSummaryDelta {
