@@ -113,8 +113,8 @@ pub enum CodexErr {
     #[error("{0}")]
     UsageLimitReached(UsageLimitReachedError),
 
-    #[error("{0}")]
-    ModelCap(ModelCapError),
+    #[error("Selected model is at capacity. Please try a different model.")]
+    ServerOverloaded,
 
     #[error("{0}")]
     ResponseStreamFailed(ResponseStreamFailed),
@@ -209,7 +209,7 @@ impl CodexErr {
             | CodexErr::Spawn
             | CodexErr::SessionConfiguredNotFirstEvent
             | CodexErr::UsageLimitReached(_)
-            | CodexErr::ModelCap(_) => false,
+            | CodexErr::ServerOverloaded => false,
             CodexErr::Stream(..)
             | CodexErr::Timeout
             | CodexErr::UnexpectedStatus(_)
@@ -409,12 +409,23 @@ impl std::fmt::Display for RetryLimitReachedError {
 pub struct UsageLimitReachedError {
     pub(crate) plan_type: Option<PlanType>,
     pub(crate) resets_at: Option<DateTime<Utc>>,
-    pub(crate) rate_limits: Option<RateLimitSnapshot>,
+    pub(crate) rate_limits: Option<Box<RateLimitSnapshot>>,
     pub(crate) promo_message: Option<String>,
+    pub(crate) limit_name: Option<String>,
 }
 
 impl std::fmt::Display for UsageLimitReachedError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        if let Some(limit_name) = self.limit_name.as_deref()
+            && !limit_name.eq_ignore_ascii_case("codex")
+        {
+            return write!(
+                f,
+                "You've hit your usage limit for {limit_name}.{}",
+                retry_suffix(self.resets_at.as_ref())
+            );
+        }
+
         if let Some(promo_message) = &self.promo_message {
             return write!(
                 f,
@@ -459,30 +470,6 @@ impl std::fmt::Display for UsageLimitReachedError {
     }
 }
 
-#[derive(Debug)]
-pub struct ModelCapError {
-    pub(crate) model: String,
-    pub(crate) reset_after_seconds: Option<u64>,
-}
-
-impl std::fmt::Display for ModelCapError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let mut message = format!(
-            "Model {} is at capacity. Please try a different model.",
-            self.model
-        );
-        if let Some(seconds) = self.reset_after_seconds {
-            message.push_str(&format!(
-                " Try again in {}.",
-                format_duration_short(seconds)
-            ));
-        } else {
-            message.push_str(" Try again later.");
-        }
-        write!(f, "{message}")
-    }
-}
-
 fn retry_suffix(resets_at: Option<&DateTime<Utc>>) -> String {
     if let Some(resets_at) = resets_at {
         let formatted = format_retry_timestamp(resets_at);
@@ -514,18 +501,6 @@ fn format_retry_timestamp(resets_at: &DateTime<Utc>) -> String {
     }
 }
 
-fn format_duration_short(seconds: u64) -> String {
-    if seconds < 60 {
-        "less than a minute".to_string()
-    } else if seconds < 3600 {
-        format!("{}m", seconds / 60)
-    } else if seconds < 86_400 {
-        format!("{}h", seconds / 3600)
-    } else {
-        format!("{}d", seconds / 86_400)
-    }
-}
-
 fn day_suffix(day: u32) -> &'static str {
     match day {
         11..=13 => "th",
@@ -589,10 +564,7 @@ impl CodexErr {
             CodexErr::UsageLimitReached(_)
             | CodexErr::QuotaExceeded
             | CodexErr::UsageNotIncluded => CodexErrorInfo::UsageLimitExceeded,
-            CodexErr::ModelCap(err) => CodexErrorInfo::ModelCap {
-                model: err.model.clone(),
-                reset_after_seconds: err.reset_after_seconds,
-            },
+            CodexErr::ServerOverloaded => CodexErrorInfo::ServerOverloaded,
             CodexErr::RetryLimit(_) => CodexErrorInfo::ResponseTooManyFailedAttempts {
                 http_status_code: self.http_status_code_value(),
             },
@@ -699,6 +671,8 @@ mod tests {
             .unwrap()
             .timestamp();
         RateLimitSnapshot {
+            limit_id: None,
+            limit_name: None,
             primary: Some(RateLimitWindow {
                 used_percent: 50.0,
                 window_minutes: Some(60),
@@ -728,8 +702,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: Some(PlanType::Known(KnownPlan::Plus)),
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -738,41 +713,11 @@ mod tests {
     }
 
     #[test]
-    fn model_cap_error_formats_message() {
-        let err = ModelCapError {
-            model: "boomslang".to_string(),
-            reset_after_seconds: Some(120),
-        };
-        assert_eq!(
-            err.to_string(),
-            "Model boomslang is at capacity. Please try a different model. Try again in 2m."
-        );
-    }
-
-    #[test]
-    fn model_cap_error_formats_message_without_reset() {
-        let err = ModelCapError {
-            model: "boomslang".to_string(),
-            reset_after_seconds: None,
-        };
-        assert_eq!(
-            err.to_string(),
-            "Model boomslang is at capacity. Please try a different model. Try again later."
-        );
-    }
-
-    #[test]
-    fn model_cap_error_maps_to_protocol() {
-        let err = CodexErr::ModelCap(ModelCapError {
-            model: "boomslang".to_string(),
-            reset_after_seconds: Some(30),
-        });
+    fn server_overloaded_maps_to_protocol() {
+        let err = CodexErr::ServerOverloaded;
         assert_eq!(
             err.to_codex_protocol_error(),
-            CodexErrorInfo::ModelCap {
-                model: "boomslang".to_string(),
-                reset_after_seconds: Some(30),
-            }
+            CodexErrorInfo::ServerOverloaded
         );
     }
 
@@ -875,8 +820,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: Some(PlanType::Known(KnownPlan::Free)),
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -889,8 +835,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: Some(PlanType::Known(KnownPlan::Go)),
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -903,8 +850,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: None,
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -921,8 +869,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: Some(PlanType::Known(KnownPlan::Team)),
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. To get more access now, send a request to your admin or try again at {expected_time}."
@@ -936,8 +885,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: Some(PlanType::Known(KnownPlan::Business)),
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -950,8 +900,9 @@ mod tests {
         let err = UsageLimitReachedError {
             plan_type: Some(PlanType::Known(KnownPlan::Enterprise)),
             resets_at: None,
-            rate_limits: Some(rate_limit_snapshot()),
+            rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
+            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -968,8 +919,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: Some(PlanType::Known(KnownPlan::Pro)),
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. Visit https://chatgpt.com/codex/settings/usage to purchase more credits or try again at {expected_time}."
@@ -978,6 +930,29 @@ mod tests {
         });
     }
 
+    #[test]
+    fn usage_limit_reached_error_hides_upsell_for_non_codex_limit_name() {
+        let base = Utc.with_ymd_and_hms(2024, 1, 1, 0, 0, 0).unwrap();
+        let resets_at = base + ChronoDuration::hours(1);
+        with_now_override(base, move || {
+            let expected_time = format_retry_timestamp(&resets_at);
+            let err = UsageLimitReachedError {
+                plan_type: Some(PlanType::Known(KnownPlan::Plus)),
+                resets_at: Some(resets_at),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
+                promo_message: Some(
+                    "Visit https://chatgpt.com/codex/settings/usage to purchase more credits"
+                        .to_string(),
+                ),
+                limit_name: Some("codex_other".to_string()),
+            };
+            let expected = format!(
+                "You've hit your usage limit for codex_other. Try again at {expected_time}."
+            );
+            assert_eq!(err.to_string(), expected);
+        });
+    }
+
     #[test]
     fn usage_limit_reached_includes_minutes_when_available() {
         let base = Utc.with_ymd_and_hms(2024, 1, 1, 0, 0, 0).unwrap();
@@ -987,8 +962,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: None,
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1096,8 +1072,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: Some(PlanType::Known(KnownPlan::Plus)),
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. Upgrade to Pro (https://chatgpt.com/explore/pro), visit https://chatgpt.com/codex/settings/usage to purchase more credits or try again at {expected_time}."
@@ -1116,8 +1093,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: None,
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1133,8 +1111,9 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: None,
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
+                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1150,10 +1129,11 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: None,
                 resets_at: Some(resets_at),
-                rate_limits: Some(rate_limit_snapshot()),
+                rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: Some(
                     "To continue using Codex, start a free trial of <PLAN> today".to_string(),
                 ),
+                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. To continue using Codex, start a free trial of <PLAN> today, or try again at {expected_time}."
