@@ -7,6 +7,7 @@ use codex_protocol::models::ContentItem;
 use codex_protocol::models::ResponseItem;
 use codex_protocol::protocol::AgentMessageContentDeltaEvent;
 use codex_protocol::protocol::AgentMessageDeltaEvent;
+use codex_protocol::protocol::AskForApproval;
 use codex_protocol::protocol::Event;
 use codex_protocol::protocol::EventMsg;
 use codex_protocol::protocol::ExitedReviewModeEvent;
@@ -91,19 +92,12 @@ async fn start_review_conversation(
         .web_search_mode
         .set(WebSearchMode::Disabled)
     {
-        tracing::warn!(
-            "failed to force review web_search_mode=disabled; falling back to a normalizer: {err}"
-        );
-        sub_agent_config.web_search_mode =
-            Constrained::normalized(WebSearchMode::Disabled, |_| WebSearchMode::Disabled)
-                .unwrap_or_else(|err| {
-                    tracing::warn!("failed to build normalizer for review web_search_mode: {err}");
-                    Constrained::allow_any(WebSearchMode::Disabled)
-                });
+        panic!("by construction Constrained<WebSearchMode> must always support Disabled: {err}");
     }
 
     // Set explicit review rubric for the sub-agent
     sub_agent_config.base_instructions = Some(crate::REVIEW_PROMPT.to_string());
+    sub_agent_config.approval_policy = Constrained::allow_only(AskForApproval::Never);
 
     let model = config
         .review_model
