@@ -2,6 +2,7 @@ pub mod apply_patch;
 pub(crate) mod collab;
 mod dynamic;
 mod grep_files;
+mod js_repl;
 mod list_dir;
 mod mcp;
 mod mcp_resource;
@@ -22,6 +23,8 @@ pub use apply_patch::ApplyPatchHandler;
 pub use collab::CollabHandler;
 pub use dynamic::DynamicToolHandler;
 pub use grep_files::GrepFilesHandler;
+pub use js_repl::JsReplHandler;
+pub use js_repl::JsReplResetHandler;
 pub use list_dir::ListDirHandler;
 pub use mcp::McpHandler;
 pub use mcp_resource::McpResourceHandler;
