@@ -42,8 +42,7 @@ mod head_tail_buffer;
 mod process;
 mod process_manager;
 
-#[cfg(any(test, feature = "test-support"))]
-pub fn set_deterministic_process_ids_for_tests(enabled: bool) {
+pub(crate) fn set_deterministic_process_ids_for_tests(enabled: bool) {
     process_manager::set_deterministic_process_ids_for_tests(enabled);
 }
 
