@@ -79,7 +79,6 @@ mod image_rollout;
 mod items;
 mod json_result;
 mod list_dir;
-mod list_models;
 mod live_cli;
 mod live_reload;
 mod model_info_overrides;
@@ -104,6 +103,7 @@ mod resume_warning;
 mod review;
 mod rmcp_client;
 mod rollout_list_find;
+mod search_tool;
 mod seatbelt;
 mod shell_command;
 mod shell_serialization;
