@@ -3,11 +3,16 @@ mod types;
 mod user_notification;
 
 pub use registry::Hooks;
+pub use registry::HooksConfig;
 pub use registry::command_from_argv;
 pub use types::Hook;
 pub use types::HookEvent;
 pub use types::HookEventAfterAgent;
+pub use types::HookEventAfterToolUse;
 pub use types::HookOutcome;
 pub use types::HookPayload;
+pub use types::HookToolInput;
+pub use types::HookToolInputLocalShell;
+pub use types::HookToolKind;
 pub use user_notification::legacy_notify_json;
 pub use user_notification::notify_hook;
