@@ -13,7 +13,6 @@
 //!
 //! Some UI is time-based rather than input-based, such as the transient "press again to quit"
 //! hint. The pane schedules redraws so those hints can expire even when the UI is otherwise idle.
-use std::collections::HashMap;
 use std::path::PathBuf;
 
 use crate::app_event::ConnectorsSnapshot;
@@ -36,11 +35,14 @@ use crossterm::event::KeyCode;
 use crossterm::event::KeyEvent;
 use ratatui::buffer::Buffer;
 use ratatui::layout::Rect;
+use ratatui::text::Line;
 use std::time::Duration;
 
 mod app_link_view;
 mod approval_overlay;
+mod multi_select_picker;
 mod request_user_input;
+mod status_line_setup;
 pub(crate) use app_link_view::AppLinkView;
 pub(crate) use approval_overlay::ApprovalOverlay;
 pub(crate) use approval_overlay::ApprovalRequest;
@@ -52,6 +54,14 @@ pub(crate) struct LocalImageAttachment {
     pub(crate) placeholder: String,
     pub(crate) path: PathBuf,
 }
+
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub(crate) struct MentionBinding {
+    /// Mention token text without the leading `$`.
+    pub(crate) mention: String,
+    /// Canonical mention target (for example `app://...` or absolute SKILL.md path).
+    pub(crate) path: String,
+}
 mod chat_composer;
 mod chat_composer_history;
 mod command_popup;
@@ -74,6 +84,8 @@ pub(crate) use feedback_view::feedback_selection_params;
 pub(crate) use feedback_view::feedback_upload_consent_params;
 pub(crate) use skills_toggle_view::SkillsToggleItem;
 pub(crate) use skills_toggle_view::SkillsToggleView;
+pub(crate) use status_line_setup::StatusLineItem;
+pub(crate) use status_line_setup::StatusLineSetupView;
 mod paste_burst;
 pub mod popup_consts;
 mod queued_user_messages;
@@ -146,7 +158,10 @@ pub(crate) struct BottomPane {
 
     /// Inline status indicator shown above the composer while a task is running.
     status: Option<StatusIndicatorWidget>,
-    /// Unified exec session summary shown above the composer.
+    /// Unified exec session summary source.
+    ///
+    /// When a status row exists, this summary is mirrored inline in that row;
+    /// when no status row exists, it renders as its own footer row.
     unified_exec_footer: UnifiedExecFooter,
     /// Queued user messages to show above the composer while a turn is running.
     queued_user_messages: QueuedUserMessages,
@@ -223,14 +238,19 @@ impl BottomPane {
         self.request_redraw();
     }
 
-    pub fn take_mention_paths(&mut self) -> HashMap<String, String> {
-        self.composer.take_mention_paths()
+    pub fn take_mention_bindings(&mut self) -> Vec<MentionBinding> {
+        self.composer.take_mention_bindings()
     }
 
-    /// Clear pending attachments and mention paths e.g. when a slash command doesn't submit text.
+    pub fn take_recent_submission_mention_bindings(&mut self) -> Vec<MentionBinding> {
+        self.composer.take_recent_submission_mention_bindings()
+    }
+
+    /// Clear pending attachments and mention bindings e.g. when a slash command doesn't submit text.
     pub(crate) fn drain_pending_submission_state(&mut self) {
         let _ = self.take_recent_submission_images_with_placeholders();
-        let _ = self.take_mention_paths();
+        let _ = self.take_recent_submission_mention_bindings();
+        let _ = self.take_mention_bindings();
     }
 
     pub fn set_steer_enabled(&mut self, enabled: bool) {
@@ -414,7 +434,7 @@ impl BottomPane {
     ///
     /// This is intended for fresh input where mention linkage does not need to
     /// survive; it routes to `ChatComposer::set_text_content`, which resets
-    /// `mention_paths`.
+    /// mention bindings.
     pub(crate) fn set_composer_text(
         &mut self,
         text: String,
@@ -432,18 +452,18 @@ impl BottomPane {
     /// Use this when rehydrating a draft after a local validation/gating
     /// failure (for example unsupported image submit) so previously selected
     /// mention targets remain stable across retry.
-    pub(crate) fn set_composer_text_with_mention_paths(
+    pub(crate) fn set_composer_text_with_mention_bindings(
         &mut self,
         text: String,
         text_elements: Vec<TextElement>,
         local_image_paths: Vec<PathBuf>,
-        mention_paths: HashMap<String, String>,
+        mention_bindings: Vec<MentionBinding>,
     ) {
-        self.composer.set_text_content_with_mention_paths(
+        self.composer.set_text_content_with_mention_bindings(
             text,
             text_elements,
             local_image_paths,
-            mention_paths,
+            mention_bindings,
         );
         self.request_redraw();
     }
@@ -476,6 +496,10 @@ impl BottomPane {
         self.composer.local_images()
     }
 
+    pub(crate) fn composer_mention_bindings(&self) -> Vec<MentionBinding> {
+        self.composer.mention_bindings()
+    }
+
     #[cfg(test)]
     pub(crate) fn composer_local_image_paths(&self) -> Vec<PathBuf> {
         self.composer.local_image_paths()
@@ -585,6 +609,7 @@ impl BottomPane {
                 if let Some(status) = self.status.as_mut() {
                     status.set_interrupt_hint_visible(true);
                 }
+                self.sync_status_inline_message();
                 self.request_redraw();
             }
         } else {
@@ -607,6 +632,7 @@ impl BottomPane {
                 self.frame_requester.clone(),
                 self.animations_enabled,
             ));
+            self.sync_status_inline_message();
             self.request_redraw();
         }
     }
@@ -637,18 +663,53 @@ impl BottomPane {
         self.push_view(Box::new(view));
     }
 
+    /// Replace the active selection view when it matches `view_id`.
+    pub(crate) fn replace_selection_view_if_active(
+        &mut self,
+        view_id: &'static str,
+        params: list_selection_view::SelectionViewParams,
+    ) -> bool {
+        let is_match = self
+            .view_stack
+            .last()
+            .is_some_and(|view| view.view_id() == Some(view_id));
+        if !is_match {
+            return false;
+        }
+
+        self.view_stack.pop();
+        let view = list_selection_view::ListSelectionView::new(params, self.app_event_tx.clone());
+        self.push_view(Box::new(view));
+        true
+    }
+
     /// Update the queued messages preview shown above the composer.
     pub(crate) fn set_queued_user_messages(&mut self, queued: Vec<String>) {
         self.queued_user_messages.messages = queued;
         self.request_redraw();
     }
 
+    /// Update the unified-exec process set and refresh whichever summary surface is active.
+    ///
+    /// The summary may be displayed inline in the status row or as a dedicated
+    /// footer row depending on whether a status indicator is currently visible.
     pub(crate) fn set_unified_exec_processes(&mut self, processes: Vec<String>) {
         if self.unified_exec_footer.set_processes(processes) {
+            self.sync_status_inline_message();
             self.request_redraw();
         }
     }
 
+    /// Copy unified-exec summary text into the active status row, if any.
+    ///
+    /// This keeps status-line inline text synchronized without forcing the
+    /// standalone unified-exec footer row to be visible.
+    fn sync_status_inline_message(&mut self) {
+        if let Some(status) = self.status.as_mut() {
+            status.update_inline_message(self.unified_exec_footer.summary_text());
+        }
+    }
+
     /// Update custom prompts available for the slash popup.
     pub(crate) fn set_custom_prompts(&mut self, prompts: Vec<CustomPrompt>) {
         self.composer.set_custom_prompts(prompts);
@@ -843,7 +904,9 @@ impl BottomPane {
             if let Some(status) = &self.status {
                 flex.push(0, RenderableItem::Borrowed(status));
             }
-            if !self.unified_exec_footer.is_empty() {
+            // Avoid double-surfacing the same summary and avoid adding an extra
+            // row while the status line is already visible.
+            if self.status.is_none() && !self.unified_exec_footer.is_empty() {
                 flex.push(0, RenderableItem::Borrowed(&self.unified_exec_footer));
             }
             let has_queued_messages = !self.queued_user_messages.messages.is_empty();
@@ -862,6 +925,18 @@ impl BottomPane {
             RenderableItem::Owned(Box::new(flex2))
         }
     }
+
+    pub(crate) fn set_status_line(&mut self, status_line: Option<Line<'static>>) {
+        if self.composer.set_status_line(status_line) {
+            self.request_redraw();
+        }
+    }
+
+    pub(crate) fn set_status_line_enabled(&mut self, enabled: bool) {
+        if self.composer.set_status_line_enabled(enabled) {
+            self.request_redraw();
+        }
+    }
 }
 
 impl Renderable for BottomPane {
@@ -1124,6 +1199,35 @@ mod tests {
         assert_snapshot!("status_only_snapshot", render_snapshot(&pane, area));
     }
 
+    #[test]
+    fn unified_exec_summary_does_not_increase_height_when_status_visible() {
+        let (tx_raw, _rx) = unbounded_channel::<AppEvent>();
+        let tx = AppEventSender::new(tx_raw);
+        let mut pane = BottomPane::new(BottomPaneParams {
+            app_event_tx: tx,
+            frame_requester: FrameRequester::test_dummy(),
+            has_input_focus: true,
+            enhanced_keys_supported: false,
+            placeholder_text: "Ask Codex to do anything".to_string(),
+            disable_paste_burst: false,
+            animations_enabled: true,
+            skills: Some(Vec::new()),
+        });
+
+        pane.set_task_running(true);
+        let width = 120;
+        let before = pane.desired_height(width);
+
+        pane.set_unified_exec_processes(vec!["sleep 5".to_string()]);
+        let after = pane.desired_height(width);
+
+        assert_eq!(after, before);
+
+        let area = Rect::new(0, 0, width, after);
+        let rendered = render_snapshot(&pane, area);
+        assert!(rendered.contains("background terminal running · /ps to view"));
+    }
+
     #[test]
     fn status_with_details_and_queued_messages_snapshot() {
         let (tx_raw, _rx) = unbounded_channel::<AppEvent>();
@@ -1228,6 +1332,7 @@ mod tests {
                 short_description: None,
                 interface: None,
                 dependencies: None,
+                policy: None,
                 path: PathBuf::from("test-skill"),
                 scope: SkillScope::User,
             }]),
